from OpenSSL import crypto
import base64
import json
from keycloak_saml_session import request

def format_pem_privatekey(key):
    """! Fonction permettant d'ajouter les entêtes
    nécéssaire au format PEM

    @param key clé privée sans les entêtes
    @return La clé privée avec les entêtes
    """
    return "-----BEGIN PRIVATE KEY-----\n"+key+"\n-----END PRIVATE KEY-----"


def load_privatekey(key):
    """! Fonction permettant de charger un clé privée

    @param key clé privée au format PEM
    @return Un objet représentant une clé privée
    """
    return crypto.load_privatekey(crypto.FILETYPE_PEM, format_pem_privatekey(key))


def sign_message(key, message, algo="sha256"):
    """! Fonction permettant de signer un message
    à partir d'une clé privée

    @param key clé privée chargée
    @param message message à signer
    @param algo Algorithme de signature
    @return Le message signé au format base64
    """
    sig = crypto.sign(key, message, algo)
    return base64.b64encode(sig)

class SessionManager:
    SESSION_EXIST = 100
    SESSION_NOT_EXIST = 101 
    def __init__(self, host, key, reaml):
        """! Fonction d'initialisation du SessionManager
        
        @param host Addresse KeyCloak
        @param key Clé privée
        @param reaml Reaml KeyCloak
        """
        self.host = host
        self.key = load_privatekey(key)
        self.reaml = reaml

    def check_session_status(self, application, id_session):
        """! Fonction permettant de checker le status d'un session
        lié à l'application

        @param application Nom de l'appllciation KeyCloak
        @param id_session Numéro de la session SAML
        @return True si le session existe, False sinon
        """
        message = {
	        "message": str(id_session)
        }
        data = json.dumps(message)
        signature = sign_message(self.key, ("SessionMessage"+data).encode())
        addons = "" if self.host[-1] == "/" else "/"
        
        url = self.host + addons + "/realms/"+ self.reaml\
            +"/saml-session-manager/" + self.reaml + "/saml/" + application
        req = request.Request(url, "POST")
        req.addHeader("saml-signature-v1", signature)
        req.addBody(data, "application/json")
        
        if req.do_request():
            if req.get_json()["exists"]:
                return SessionManager.SESSION_EXIST
            else:
                return SessionManager.SESSION_NOT_EXIST
        else:
            return None



import requests

class MalformatedUrl(Exception):
    pass


class UnknownMethod(Exception):
    pass


class BadMethod(Exception):
    pass


class ParamsRequest:
    def __init__(self):
        self.param = {}

    def addParam(self, key, value):
        self.param[key] = value

    def build(self):
        return self.param

    def build_url(self):
        if len(self.param) == 0:
            return ""
        url = ""
        count = 0
        for key in self.param:
            if count == 0:
                url = url + key + "=" + self.param[key]
            else:
                url = url + "&" + key + "=" + self.param[key]
        return "?" + url


class Request:
    methods = ["GET", "POST", "PUT", "PATCH", "DELETE"]
    call_methods = {
        "GET": requests.get,
        "POST": requests.post,
        "PUT": requests.put,
        "PATCH": requests.patch,
        "DELETE": requests.delete
    }

    def __init__(self, url, method, bearer=None):
        self.url = url
        self.method = method
        self.call_method = Request.call_methods[self.method]
        self.param = {}
        self.headers = {}
        self.body = ""
        self.bodyMimeType = ""
        self.response = None
        self.bearer_token = bearer
        self.check_method = lambda c: (c >= 200 and c <= 399)

    def addParam(self, key, value):
        self.param[key] = value

    def addParamObject(self, param):
        self.param = param.build()

    def addBody(self, value, mimeType):
        if self.method not in ["POST", "PUT", "PATCH"]:
            raise BadMethod(self.method)
        self.body = value
        self.bodyMimeType = mimeType

    def addHeader(self, key, value):
        self.headers[key] = value

    def do_request(self):
        if self.bearer_token is not None:
            self.headers["Authorization"] = "Bearer " + self.bearer_token
        if self.bodyMimeType != "":
            self.headers["Content-type"] = self.bodyMimeType
        self.response = self.call_method(self.url,
                                         params=self.param,
                                         data=self.body,
                                         headers=self.headers)
        return self.check_response()

    def get_raw(self):
        if self.response is None or self.check_empty():
            return None
        return self.response.text

    def get_json(self):
        if self.response is None or self.check_empty():
            return None
        return self.response.json()

    def build_url(self):
        uri = []
        for el in self.param:
            uri.append(el+"="+str(self.param[el]))
        if len(uri) > 0:
            if self.url[-1] == "/":
                return self.url + "?" + "&".join(uri)
            return self.url + "/?" + "&".join(uri)
        else:
            return self.url

    def check_response(self):
        return self.check_method(self.response.status_code)

    def check_empty(self):
        return self.response.text == ""
